package org.gfxtest.framework;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

public class TestImage
{
    private Log log = null;
    private static final int GRID_SIZE = 20;
    private BufferedImage image = null;

    public TestImage(Configuration configuration)
    {
        log = new Log(this.getClass().getName(), true);
        this.createImage(configuration);
    }

    /**
     * Create new buffered image with given width, height and image type.
     * @param configuration configuration of current test suite
     * @return
     */
    public void createImage(Configuration configuration)
    {
        this.image = new BufferedImage(configuration.getImageWidth(), configuration.getImageHeight(), configuration.getImageType());
        this.clearImage();
        this.drawGrid();
    }

    protected void clearImage()
    {
        fillImage(Color.WHITE);
    }

    public void fillImage(Color color)
    {
        Graphics2D graphics = this.image.createGraphics();
        graphics.setColor(color);
        graphics.fillRect(0, 0, this.image.getWidth(), this.image.getHeight());
        graphics.dispose();
    }

    protected BufferedImage readImage(File directory, String fileName) throws IOException
    {
        log.log("reading image " + fileName + " from directory " + directory);
        File imageFile = new File(directory, fileName);
        return ImageIO.read(imageFile);
    }

    private void writeImage(File directory, String fileName) throws IOException
    {
        log.log("writing image " + fileName + " into directory " + directory);
        ImageIO.write(this.image, "png", new File(directory, fileName));
    }

    /**
     * Write given buffered image to PNG file.
     *
     * @param image
     * @param configuration
     * @param testName
     * @throws IOException
     */
    public void writeImage(Configuration configuration, String suiteName, String testName) throws IOException
    {
        this.writeImage(configuration.getOutputPath(), suiteName + "_" + testName + ".png");
    }

    /**
     * This method draws grid below the test (rendered) pattern.
     * @param image reference to the image created by test case
     */
    private void drawGrid()
    {
        Graphics2D graphics = this.image.createGraphics();
        graphics.setColor(new Color(0xa0, 0xa0, 0xff));
        for (int x = 0; x < image.getWidth(); x += GRID_SIZE)
        {
            graphics.drawLine(x, 0, x, image.getHeight() - 1);
        }
        for (int y = 0; y < image.getHeight(); y += GRID_SIZE)
        {
            graphics.drawLine(0, y, image.getWidth() - 1, y);
        }
        graphics.dispose();
    }

    public Graphics2D getGraphics()
    {
        return (Graphics2D) this.image.getGraphics();
    }

    public int getWidth()
    {
        return this.image.getWidth();
    }

    public int getHeight()
    {
        return this.image.getHeight();
    }

}
